C This subroutine extracts the roots of the polynome c2*z^2+c1*z+c0=0
C with the respective values zm and zp for the -/+ depending on the sign
C of the squareroot. The coefficients 'c' are complex
C This file is part of aITALC
C Copyright (C) 2004 Alejandro Lorca <alejandro.lorca@desy.de>
C-----------------------------------------------------------------------
      SUBROUTINE CGETROOTS(c2,c1,c0,zm,zp)
      implicit none
      double complex c2,c1,c0
      double complex zm,zp,ztrans
c      integer index
      double complex czero
      double complex term1,term2,expan,theroot,x,rc1,rc0
      parameter (czero=(0d0,0d0))
C-----------------------------------------------------------------------
c      print *,'Coeff c0=',c0
c      print *,'Coeff c1=',c1
c      print *,'Coeff c2=',c2
C-----------------------------------------------------------------------
C Trivial case with a2=0
      if (c2.EQ.czero) then
        print *,'cgetroots: error, coefficient c2 of z^2 = 0'
        stop
      endif
C-----------------------------------------------------------------------
C Trivial case with c0=0
      if (c0.EQ.czero) then
        if (c1.EQ.czero) then
          zm=czero
          zp=zm
c          index=0
          goto 20
        endif
        zp=DCMPLX(-c1/c2)
        zm=czero
c        print *,'c0=0'
c        index=1
        goto 20
      endif
C-----------------------------------------------------------------------
C Trivial case with c1=0

      if (c1.EQ.czero) then
        zp=+SQRT(-c0/c2)
        zm=-zp
c        index=1
        goto 20
      endif
C-----------------------------------------------------------------------
C Non-trivial case.
      term1=c1**2
      term2=4*c2*c0
      x=term2/term1
      theroot=SQRT(term1-term2)
      expan=x/2*(
     &         +1
     &         +x/4
     &         +x**2/8
     &         )
      if (ABS(1-x).LT.2d-15) then
c        print *,'the same roots'
C Sqrt=0, same roots, within 2d-15 security round-offset
        zp=DCMPLX(-c1/(2*c2))
        zm=zp
c        index=0
        goto 20
      else
        zp=-c1+theroot
        zm=-c1-theroot
        if (ABS(x).LT.3D-4) then
C Since we don't know which is the accurate yet
C and now we compare them!

          if (ABS(zp).LT.ABS(zm)) then
            zp=-c1*expan
          else
            zm=-c1*expan
          endif
c         print *,'cgetroots: expansion performed, some acc recovered'
c         print *,'zp=',zp
c         print *,'zm=',zm
        endif
        zp=zp/(2*c2)
        zm=zm/(2*c2)
        goto 20
      endif
C-----------------------------------------------------------------------
 20   continue
C Ordering; the zm is expected to be lower also if sign(c2) < 0
      if (DBLE(zp).LT.DBLE(zm)) then
C Do the change,
        ztrans=zp
        zp=zm
        zm=ztrans
      endif
C C Reconstruction test
      rc0=+c2*(zp*zm)
      rc1=-c2*(zp+zm)
      if ((c0.EQ.czero).AND.(rc0.NE.czero)) then
        print *,'cgetroots: error on c0 reconstruction'
     &       ,ABS(rc0-c0)
        print *,'c0=',c0,rc0   
        goto 30
      endif
      if ((c1.EQ.czero).AND.(rc1.NE.czero)) then
        print *,'cgetroots: error on c1 reconstruction'
     &       ,ABS(rc1-c1)
        print *,'c1=',c1,rc1
        goto 30
      endif
      if (ABS((rc0-c0)/c0).GT.3D-12) then
        print *,'cgetroots: error on c0 reconstruction'
     &       ,ABS((rc0-c0)/c0)
        print *,'c0=',c0,rc0
        stop
      elseif (ABS((rc1-c1)/c1).GT.3D-12) then
        print *,'cgetroots: error on c1 reconstruction'
     &       ,ABS((rc1-c1)/c1)
        print *,'c1=',c1,rc1
        stop
      endif
C-----------------------------------------------------------------------
 30   return
      end

C $Id: cgetroots.f,v 1.4 2004/10/15 19:19:06 alorca Exp $
